#include "main.h"
#include "usart.h"
#include "dcf77.h"

//Steuerregister Timer
//volatile uint16_t ui_tim0cnt;
//volatile uint8_t ui_tim0sreg;
//volatile uint16_t ui_tim1cnt;
volatile uint16_t ui_tim1cnt_sec;	//Sekunden Counter


//Timebase - 1/1000-Sekunde
#define TIMXDiv 1000
#define TIMXVal (F_CPU/TIMXDiv)
#define TIMXRem (F_CPU%TIMXDiv)

//Steuerregister Delay-Funktion "fwait"
volatile uint8_t ui_fwait = 0;	//Steuerregister
volatile uint16_t ui_fwait_delay = 0;

//Globale Variablen DCF-Empfang
volatile long long dc_buffer;
volatile int dc_buffer_counter;
volatile uint8_t ui_dcf_cnt = 0;	//Timer, der prft, ob DCF Flanke kommt
volatile uint8_t ui_dcfsreg;		//Steuerregister fr DCF-Empfang!!!
									//->B0	Zeitinformationen liegen vor
									//->B1	Timer Start/Stopp
									//->B2	Fehlendes Signal fr ein paar Sekunden festgestellt
									//->B3	Parittsfehler
volatile uint16_t ui_dcf;


//Uhrzeit - DCF-Signal
volatile unsigned char mm	= 0;	//Globale Variable fr die Minuten
volatile unsigned char hh	= 0;	//Globale Variable fr die Stunden
volatile unsigned char day	= 0;	//Globale Variable fr den Tag
volatile unsigned char mon	= 0;	//Globale Variable fr den Monat
volatile unsigned int year	= 0;	//Globale Variable fr das Jahr
//Uhrzeit - manuelle Uhr
volatile unsigned char manclock_ss	= 0;	//Globale Variable fr die Sekunden
volatile unsigned char manclock_mn = 0;
volatile unsigned char manclock_hr = 0;

void fwait(uint16_t ui_delay)
{
//Wartefunktion in Millisekunden
	ui_fwait |= (1<<0);			//BIT 0 setzen

	while(ui_fwait_delay < ui_delay)
	{
		asm("nop");
	}

	ui_fwait &= ~(1<<0);			//BIT 0 lschen
	ui_fwait_delay = 0;
}

int main(void)
{
//Interrupt Eingang

	DDRD |= 0b00000000;
	PORTD |= 0b00000100;
	MCUCR |= (0<<ISC01) | (1<<ISC00);	//Jede nderung
	GICR |= (1<<INT0);
	GIFR |= (1<<INTF0);

//Timer 1 - test fr Timebase
	
	DDRB |= 0b00000010;					//Kanal A -> wechselt Zustand bei jedem Match
										//Daher Port Kanal A als Ausgang setzen
	OCR1A = TIMXVal;					//Vergleichsregister Kanal A bekomme Value
	TCCR1A |= (1<<COM1A0);				//Kanal A -> umschalten bei Match (TOGGLE)
	TCCR1B |= (1<<WGM12) | (1<<CS10);	//Prescaller = 1
	TIMSK |= (1<<OCIE1A);				//Match-Interrupt aktivieren
	
	sei();								//Interrupts aktivieren
	
	usart_init();						//USART initialisieren

//Main Loop
	
	while(1) 
	{
		fwait(1000);
		
		dcf77_out();
		
	}	
	return 0;
}


SIGNAL(SIG_OUTPUT_COMPARE1A)
{
	ui_tim1cnt_sec++;
	if(ui_tim1cnt_sec == TIMXDiv)		//Eine Sekunde vergangen
	{
		//usart_senddata('#');
		manclock_ss++;							//Sekunden incrementierten
		
		ui_dcf_cnt++;					//DCF-Signal Kontroll-Register incrementieren
		
		if(manclock_ss==60)						//Prfen ob Sek. Max erreicht
		{
			manclock_ss = 0;						//Sekunden zurcksetzen
			manclock_mn ++;				//Minuten der Manuellen Uhr incrementierten
			if(manclock_mn == 60)		//Prfen ob Minuten Max erreicht
			{		
				manclock_mn = 0;		//Minuten zurcksetzen
				manclock_hr ++;			//Stunden incrementieren
				if(manclock_hr == 24)	//Prfen ob Stunden Max erreicht
				{
					manclock_hr = 0;	//Stunden zurcksetzen
				}
			}
		}
		ui_tim1cnt_sec=0;				//Neuer Zyklus fr Compare Match
										//Diese Variable wird auch in DCF-Empfangsroutine
										//gelscht, um Sekunden Impuls zu synchronisieren.
	}

//Strung durch fehlendes Signal melden
	if(ui_dcf_cnt >= 3)
	{
		ui_dcfsreg |= (1 << 2);			//Fehlendes Signal signalisieren
		ui_dcfsreg |= (1 << 3);			//Parity-Fehler ebenfalls setzen
		dc_buffer = 0;	 				//DC Buffer lschen
	}
	else
	{
		//Fehler des fehlendes Signal nur rcksetzen, wenn Parity OK
		if(!(ui_dcfsreg & (1 << 3)))
		{
			ui_dcfsreg &= ~(1 << 2);		//Fehlendes Signal signalisieren
		}
	}
	
//Timer fr DCF-Empfang
//hiermit wird die Pulsweite des empfangenen Signals ausgewertet
	if(ui_dcfsreg &= (1<<1))
	{
		ui_dcf++;
	}

//Timer fwait
	if(ui_fwait &= (1<<0))
	{
		ui_fwait_delay ++;
	}
}

SIGNAL(SIG_OUTPUT_COMPARE1B)
{
	usart_senddata('B');
}

/////////////////////////////////////////////////////////////////////
//	Signal handler for dcf_signal
//	
//13.08 
//BIT Counter muss zur berwachung noch eingefgt werden -> sprich
// kommen alle Bits auch wirklich an vom DCF Signal!???
SIGNAL(SIG_INTERRUPT0)
{
	ui_dcfsreg &= ~(1<<1);	//DCF Zhler AUS
	uint16_t myui = ui_dcf;
	ui_dcf=0;
	ui_dcfsreg |= (1<<1);	//DCF Zhler EIN
	uint8_t result=9;
	
	ui_dcf_cnt = 0;	//Wird bei jedem Impuls zurckgesetzt!
					//Ist dieses Register > 2 entspr. > 2 Sekunden
					//so wird ein Fehler im Empfang gemeldet
					//-> Hat zur Folge, dass manuelle Uhr ausgegeben wird!
					//Zurckgesetzt wird es erst bei passenden Signal
	
	if((myui > 785) & (myui < 815))
	{
		//ca. 800ms pause -> Absenkung 200ms == Logisch 1
		result = 1;
	}

	if((myui > 885) & (myui < 915))
	{
		//ca. 900ms pause -> Absenkung 100ms == Logisch 0
		result = 2;
	}

	if(myui > 1000)
	{
		//In der 59Sekunde erfolgt keine Trgerabsenkung
		result = 3;
	}

	switch(result)
	{
		case 1:
			dc_buffer_counter++;
			dc_buffer |= ((unsigned long long) 1 << dc_buffer_counter);
			break;
		case 2:
			dc_buffer_counter++;
			dc_buffer &= ~((unsigned long long) 1 << dc_buffer_counter);
			break;
		case 3:
			dc_buffer_counter=-1;		//Von vorn beginnen
			//Die Sekunden werden nur synchronisiert, wenn kein Fehler vorliegt!!!
			if(!(ui_dcfsreg & (1 << 2)) && !(ui_dcfsreg & (1 << 3)))
			{
				manclock_ss = 0;						//Sekunden synchronisieren
				ui_tim1cnt_sec = 0;			//SekundenCounter zurckstellen (zhlt 1000ms)
			}
			ui_dcfsreg |= (1 << 0);			//Bekanntmachung aktueller Zeitinformationen
			break;
		default:
			break;
	}
}
