/////////////////////////////////////////////////
//	usart.c
//	
//	Serielle Kommunikation
//	9600baud 8n1
//
//	Letzte nderung:	16.08.2006
//						Erstversion
//
/////////////////////////////////////////////////
//
#include "main.h"
#include "usart.h"
// 
// Hilfsmakro zur UBRR-Berechnung ("Formel" laut Datenblatt)
#define UART_UBRR_CALC(BAUD_,F_CPU) ((F_CPU)/((BAUD_)*16L)-1)
#define UART_BAUD_RATE 9600

void usart_init(void)
{
	//Uart einstellen
 	UBRRH = (uint8_t)(UART_UBRR_CALC(UART_BAUD_RATE,F_CPU)>>8);
 	UBRRL = (uint8_t)UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);

	//Initialisiert den UART jedoch ohne Empfangsinterrupt
	//Format einstellen (8N1)
	UCSRC |= (1<<URSEL)|(3<<UCSZ0);
	//Senden und Empfangen einschalten
	UCSRB |= (1<<RXEN) | (1<<TXEN);
}

/////////////////////////////////////////////////
//	usart_string(unsigned char)
//	
//	16.08.2006	Postler
//	gibt eine Zeichenkette aus
//
/////////////////////////////////////////////////
void usart_string(unsigned char *uc)
{
	uint8_t i = 0;
	while(uc[i])
	{
		usart_senddata(uc[i]);
		i++;
	}
}

/////////////////////////////////////////////////
//	usart_senddata(unsigned char)
//	
//	16.08.2006	Postler
//	sendet ein Zeichen via USART
//
/////////////////////////////////////////////////
void usart_senddata(unsigned char data)
{
	while( !(UCSRA & (1<<UDRE)) )
	{
	
	}
		UDR = data;
}

/////////////////////////////////////////////////
//	usart_int16str(uint16_t)
//	
//	16.08.2006	Postler
//	gibt einen 16-BIT Integer als String aus
//
/////////////////////////////////////////////////
void usart_int16str(uint16_t uiq)
{
	unsigned char mybuf[5];
	
	itoa(uiq,mybuf,10);
	usart_string(mybuf);
}

/////////////////////////////////////////////////
//	usart_niblebcd(uint8_t)
//	
//	16.08.2006	Postler
//	gibt die unteren vier bits eines byte 
//	bcd-codiert aus
//
/////////////////////////////////////////////////
void usart_niblebcd(uint8_t ui_data)
{
	ui_data &= 0x0F;
	ui_data |= 0x30;
	usart_senddata(ui_data);
}

/////////////////////////////////////////////////
//	usart_bytebcd(uint8_t)
//	
//	16.08.2006	Postler
//	gibt ein byte bcd-codiert aus
//
/////////////////////////////////////////////////
void usart_bytebcd(uint8_t ui_data)
{
	//als erstes High-Nibble ausgeben
	usart_niblebcd(ui_data >> 4);
	usart_niblebcd(ui_data);

}

/////////////////////////////////////////////////
//	usart_bytebin(uint8_t)
//	
//	16.08.2006	Postler
//	gibt ein byte binr aus
//
/////////////////////////////////////////////////
void usart_bytebin(uint8_t ui_binout)
{
	uint8_t ui_cnt;
	for(ui_cnt = 0; ui_cnt <= 7; ui_cnt++)
	{			
		if(ui_binout & (1 << (7-ui_cnt)))
		{
			usart_senddata('1');
		}
		else
		{
			usart_senddata('0');
		}
	}
}
